#pragma once

#include <common/common.h>
#include <lektor/mkv.h>
#include <lektor/config.h>
#include <lektor/window.h>
#include <lektor/thread.h>
#include <lektor/uri.h>

#include <sqlite3.h>
#include <mpv/client.h>
#include <curl/curl.h>
#include <sqlite3.h>
#include <json-c/json.h>
#include <inttypes.h>
#include <stdbool.h>

#include <stddef.h>

struct lkt_command {
    char *name;
    char *args[LKT_MESSAGE_ARGS_MAX];
    long cont;
};

/* Repository, e.g. Kurisu */
struct lkt_repo {
    /* Just the repo */
    const char *name;
    const char *base_url;
    const char *kara_dir;
    const char *get_all_json;
    const char *get_id_json;
    const char *get_id_file;
    const uint64_t version;

    /* The database */
    volatile sqlite3 *db;
};

int  repo_new (struct lkt_repo *const repo, const char *name, const char *url, volatile sqlite3 *db);
void repo_free(struct lkt_repo *const repo);

int repo_get_id          (struct lkt_repo *const repo, const uint64_t id, struct kara_metadata *mdt);
int repo_update          (struct lkt_repo *const repo);
int repo_download_id_sync(struct lkt_repo *const repo, const uint64_t id, const char *kara_path, struct kara_metadata *mdt);

/* Create and destruct a command structure. */
struct lkt_command lkt_command_parse(char *raw);
struct lkt_command lkt_command_clone(struct lkt_command *c);
void lkt_command_free(struct lkt_command *c);

/* Messages that are send and recieved */
struct lkt_message {
    size_t ref_count;
    char data[LKT_MESSAGE_MAX];
    size_t data_len;
};

/* Create and destruct a message structure. */
struct lkt_message *lkt_message_new();
void lkt_message_free(struct lkt_message *msg);
struct lkt_message *lkt_message_clone(struct lkt_message *msg);

struct lkt_state {
    struct pollfd *fds;
    struct lkt_client *clients;
    size_t fds_len;
    size_t fds_max;

    volatile sqlite3 *db;
    const char *kara_prefix;

    struct lkt_repo repo;

    mpd_idle_flag mpd_idle_events;

    struct lkt_win win;
};

/* Send a message to the connected client. */
void lkt_state_send(struct lkt_state *srv, size_t c, struct lkt_message *msg);

/* Get and set continuation state for a client. */
size_t lkt_remaining_msg(struct lkt_state *srv, size_t c);

int  lkt_get_continuation(struct lkt_state *srv, size_t c);
void lkt_set_continuation(struct lkt_state *srv, size_t c, int i);

/* Get the mask to watch for events for a given client, to be used with the `idle` command.
 * Return a pointer to the client's mask */
mpd_idle_flag *lkt_client_get_mask(struct lkt_state *srv, size_t c);

/* Register a client as authentificated. */
bool lkt_client_auth(struct lkt_state *srv, size_t c, bool set_auth);

/* The server's listen function. */
int lkt_listen(void);

