#pragma once

/* Types */
typedef volatile unsigned int mthread_tst_t;

struct mthread_list_s;
typedef struct mthread_list_s mthread_list_t;

struct mthread_s;
typedef struct mthread_s *mthread_t;

typedef enum {
    ATTR_NONE           = 0,
    ATTR_DETACHED       = (1 << 1),
    ATTR_DETACHED_FREE  = (1 << 2),
} mthread_attr_t;

struct mthread_mutex_s {
    volatile int nb_thread;
    mthread_tst_t lock;
    mthread_list_t *list;   /* lock management already handled */
};
typedef struct mthread_mutex_s mthread_mutex_t;

#define MTHREAD_MUTEX_INITIALIZER { .nb_thread = 0, .lock = 0, .list = NULL }

struct mthread_mutexattr_s;
typedef struct mthread_mutexattr_s mthread_mutexattr_t;

struct mthread_cond_s {
    mthread_tst_t lock;         /* Protects the structure.  */
    mthread_list_t *list;       /* List of waiting threads. */
};
typedef struct mthread_cond_s mthread_cond_t;

struct mthread_condattr_s;
typedef struct mthread_condattr_s mthread_condattr_t;

typedef unsigned int mthread_key_t;

typedef mthread_tst_t mthread_once_t;

struct mthread_sem_s {
    volatile unsigned int value;    /* Number of wait that are done at the same time. */
    mthread_mutex_t mutex;          /* Protect max_permit.                            */
};
typedef struct mthread_sem_s mthread_sem_t;

/* Function for handling threads.  */

extern int mthread_create(mthread_t *__threadp, const mthread_attr_t __attr, void *(*__start_routine) (void *), void *__arg);

extern mthread_t mthread_self(void);
extern int  mthread_equal (mthread_t __thread1, mthread_t __thread2);
extern void mthread_exit  (void *__retval);
extern int  mthread_join  (mthread_t __th, void **__thread_return);

/* Functions for mutex handling.  */

extern int mthread_mutex_init   (mthread_mutex_t *__mutex, const mthread_mutexattr_t *__mutex_attr);
extern int mthread_mutex_destroy(mthread_mutex_t *__mutex);
extern int mthread_mutex_trylock(mthread_mutex_t *__mutex);
extern int mthread_mutex_lock   (mthread_mutex_t *__mutex);
extern int mthread_mutex_unlock (mthread_mutex_t *__mutex);

/* Functions for handling conditional variables.  */

extern int mthread_cond_init     (mthread_cond_t *__cond, const mthread_condattr_t *__cond_attr);
extern int mthread_cond_destroy  (mthread_cond_t *__cond);
extern int mthread_cond_signal   (mthread_cond_t *__cond);
extern int mthread_cond_broadcast(mthread_cond_t *__cond);
extern int mthread_cond_wait     (mthread_cond_t *__cond, mthread_mutex_t *__mutex);

/* Functions for handling thread-specific data.  */

extern int  mthread_key_create  (mthread_key_t *__key, void (*__destr_function) (void *));
extern int  mthread_key_delete  (mthread_key_t __key);
extern int  mthread_setspecific (mthread_key_t __key, const void *__pointer);
extern void *mthread_getspecific(mthread_key_t __key);


/* Functions for handling initialization.  */

/* Guarantee that the initialization function INIT_ROUTINE will be called
   only once, even if mthread_once is executed several times with the
   same ONCE_CONTROL argument. ONCE_CONTROL must point to a static or
   variable initialized to MTHREAD_ONCE_INIT.

   The initialization functions might throw exception which is why
   this function is not marked with .  */
extern int mthread_once(mthread_once_t *__once_control, void (*__init_routine) (void));

/* Functions for handling semaphore.  */

extern int mthread_sem_init(mthread_sem_t *sem, unsigned int value);
extern int mthread_sem_wait(mthread_sem_t *sem);     /* P(sem), wait(sem) */
extern int mthread_sem_post(mthread_sem_t *sem);     /* V(sem), signal(sem) */

extern int mthread_sem_getvalue(mthread_sem_t *sem, int *sval);
extern int mthread_sem_trywait (mthread_sem_t *sem);
extern int mthread_sem_destroy (mthread_sem_t *sem);  /* undo sem_init() */

extern void mthread_yield();

/* Initialize mthread. */

extern void mthread_init(void);
extern void *mthread_main(void *arg);
