#define _POSIX_C_SOURCE 200809L

#include <common/common.h>
#include <lektor/uri.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <errno.h>
#include <limits.h>

bool
lkt_uri_from(struct lkt_uri *ret, char *const str)
{
    char *val, *endptr;
    size_t len;
    long id;

    if (ret == NULL || str == NULL)
        return false;

    len = strlen(str);

    // The minimal uri is id://1 which is a 6 characters long
    if (len <= 5 * sizeof(char))
        return false;

    if (!strncasecmp(str, "fs", 2 * sizeof(char))) {
        ret->type = uri_fs;
        val = str + 2;
    } else if (!strncasecmp(str, "id", 2 * sizeof(char))) {
        ret->type = uri_id;
        val = str + 2;
    } else if (!strncasecmp(str, "plt", 3 * sizeof(char))) {
        ret->type = uri_playlist;
        val = str + 3;
    } else if (!strncasecmp(str, "type", 4 * sizeof(char))) {
        ret->type = uri_type;
        val = str + 4;
    } else if (!strncasecmp(str, "author", 6 * sizeof(char))) {
        ret->type = uri_author;
        val = str + 6;
    } else if (!strncasecmp(str, "category", 8 * sizeof(char))) {
        ret->type = uri_category;
        val = str + 8;
    } else if (!strncasecmp(str, "lang", 4 * sizeof(char))) {
        ret->type = uri_language;
        val = str + 4;
    } else if (!strncasecmp(str, "query", 5 * sizeof(char))) {
        ret->type = uri_query;
        val = str + 5;
    } else
        return false;

    // No place for the '://' separator
    if (len - (val - str) < 3)
        return false;

    val += 3;

    if (ret->type == uri_id) {
        id = strtol(val, &endptr, 10);

        if ((errno == ERANGE && (id == LONG_MAX || id == LONG_MIN))
            || (errno != 0 && id == 0) || endptr == str)
            return false;

        ret->value = calloc(1, sizeof(int));

        if (ret->value == NULL)
            return false;

        *(int *) ret->value = id;
        ret->_allocated = true;
    }

    else {
        ret->_allocated = false;
        ret->value = val;
    }

    return true;
}

void
lkt_uri_free(struct lkt_uri *ret)
{
    if (NULL == ret)
        return;

    if (ret->_allocated)
        free(ret->value);
}

/* Support a subrange of URIs. */
static inline int
__lkt_to_str(struct lkt_uri *uri, char *ret, size_t len)
{
    switch (uri->type) {
    case uri_id:
        safe_snprintf(ret, len, "id=%s", (char *) uri->value);
        break;
    case uri_type:
        safe_snprintf(ret, len, "type=%s", (char *) uri->value);
        break;
    case uri_author:
        safe_snprintf(ret, len, "author=%s", (char *) uri->value);
        break;
    case uri_category:
        safe_snprintf(ret, len, "cat=%s", (char *) uri->value);
        break;
    case uri_query:
        safe_snprintf(ret, len, "search=%s", (char *) uri->value);
        break;
    default:
        LOG_ERROR("URI type %d may not be supported by kurisu, generate an error", uri->type);
        return 1;
    }

    ret[len - 1] = '\0';
    return 0;
}

int
lkt_uri_to_url(struct lkt_uri *uri, const char *base_url, char *ret, size_t len)
{
    RETURN_UNLESS(uri && base_url && ret && len > 0, "Invalid argument", 1);

    /* Craft URL */
    memset(ret, 0, len);
    memcpy(ret, base_url, sizeof(char) * strlen(base_url));
    strncat(ret, "?", len - 1);

    /* The query */
    size_t init_len = strlen(ret);
    return __lkt_to_str(uri, ret + init_len, len - init_len);
}
